
/* 
 * Title       : own.c
 * Description : 1-Wire subroutines
 * Author      : KAWAMOTO Yasuhisa
 * URL         : http://www.devdrv.co.jp/
 * Date        : 01/12/2006
 */
 
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <c8051f330.h>
#include "main.h"

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------
//
// OWTouchReset timing
//  <-A-><-  B  -><- C -><-  D  ->  
//  ~~~~\        /~~~\~~S~~~~~~~~/~~
//       ~~~~~~~~     ~~ ~~~~~~~~   
#define DELAY_A  2000*(SYSCLK/100000)/140	// 2000us(Parasitic power is supplied to DQ)
#define DELAY_B   480*(SYSCLK/100000)/140	//  480us
#define DELAY_C    70*(SYSCLK/100000)/140	//   70us
#define DELAY_D   410*(SYSCLK/100000)/140	//  410us
//
// OWTouchBit timing
//      <-E-><-F-><-  G  -><-H->    
//  ~~~\    /~~~~S~~~~~~~~/~~~~\    
//      ~~~~ ~~~~ ~~~~~~~~      ~~~~
#define DELAY_E    6*(SYSCLK/100000)/140	//   6us
#define DELAY_F    9*(SYSCLK/100000)/140	//   9us
#define DELAY_G   45*(SYSCLK/100000)/140	//  45us
#define DELAY_H   10*(SYSCLK/100000)/140	//  10us

//-----------------------------------------------------------------------------
// Global VARIABLES
//-----------------------------------------------------------------------------
unsigned char ROMCode[8];
unsigned char ScratchPAD[9];
unsigned char LastDiscrepancy;
unsigned char LastFamilyDiscrepancy;
unsigned char LastDeviceFlag;
unsigned char crc8;
const unsigned char dscrc_table[256] =
{
	  0,  94, 188, 226,  97,  63, 221, 131, 194, 156, 126,  32, 163, 253,  31,  65,
	157, 195,  33, 127, 252, 162,  64,  30,  95,   1, 227, 189,  62,  96, 130, 220,
	 35, 125, 159, 193,  66,  28, 254, 160, 225, 191,  93,   3, 128, 222,  60,  98,
	190, 224,   2,  92, 223, 129,  99,  61, 124,  34, 192, 158,  29,  67, 161, 255,
	 70,  24, 250, 164,  39, 121, 155, 197, 132, 218,  56, 102, 229, 187,  89,   7,
	219, 133, 103,  57, 186, 228,   6,  88,  25,  71, 165, 251, 120,  38, 196, 154,
	101,  59, 217, 135,   4,  90, 184, 230, 167, 249,  27,  69, 198, 152, 122,  36,
	248, 166,  68,  26, 153, 199,  37, 123,  58, 100, 134, 216,  91,   5, 231, 185,
	140, 210,  48, 110, 237, 179,  81,  15,  78,  16, 242, 172,  47, 113, 147, 205,
	 17,  79, 173, 243, 112,  46, 204, 146, 211, 141, 111,  49, 178, 236,  14,  80,
	175, 241,  19,  77, 206, 144, 114,  44, 109,  51, 209, 143,  12,  82, 176, 238,
	 50, 108, 142, 208,  83,  13, 239, 177, 240, 174,  76,  18, 145, 207,  45, 115,
	202, 148, 118,  40, 171, 245,  23,  73,   8,  86, 180, 234, 105,  55, 213, 139,
	 87,   9, 235, 181,  54, 104, 138, 212, 149, 203,  41, 119, 244, 170,  72,  22,
	233, 183,  85,  11, 136, 214,  52, 106,  43, 117, 151, 201,  74,  20, 246, 168,
	116,  42, 200, 150,  21,  75, 169, 247, 182, 232,  10,  84, 215, 137, 107,  53
};

//-----------------------------------------------------------------------------
// tickDelay
//-----------------------------------------------------------------------------
void tickDelay(unsigned short t)
{
	while(t--);
}

//-----------------------------------------------------------------------------
// OWTouchReset
//-----------------------------------------------------------------------------
unsigned char OWTouchReset(void)
{
	unsigned char result = FALSE;
	
	EA = 0;
	
	tickDelay(DELAY_A);
	OWN_DRIVE;
	tickDelay(DELAY_B);
	OWN_RELEASE;
	tickDelay(DELAY_C);
	if(OWN_SENSE == 0) result = TRUE;
	tickDelay(DELAY_D);
	
	EA = 1;
	
	return(result);
}

//-----------------------------------------------------------------------------
// OWTouchBit
//-----------------------------------------------------------------------------
unsigned char OWTouchBit(unsigned char bit_value)
{
	unsigned char result;
	
	EA = 0;
	
	OWN_DRIVE;
	tickDelay(DELAY_E);
	if(bit_value)
	{
		OWN_RELEASE;
	}
	else
	{
		OWN_DRIVE;
	}
	tickDelay(DELAY_F);
	result = OWN_SENSE;
	tickDelay(DELAY_G);
	OWN_RELEASE;
	tickDelay(DELAY_H);
	
	EA = 1;
	
	return(result);
}

//-----------------------------------------------------------------------------
// OWTouchByte
//-----------------------------------------------------------------------------
unsigned char OWTouchByte(unsigned char byte_value)
{
	unsigned char i;
	unsigned char result = 0;
	
	for(i = 0; i < 8; i++)
	{
		result >>= 1;
		
		if(OWTouchBit(byte_value & 0x01))
			result |= 0x80;
		
		byte_value >>= 1;
	}
	
	return(result);
}

//-----------------------------------------------------------------------------
// OWWriteBit
//-----------------------------------------------------------------------------
void OWWriteBit(unsigned char bit_value)
{
	OWTouchBit(bit_value);
}

//-----------------------------------------------------------------------------
// OWReadBit
//-----------------------------------------------------------------------------
unsigned char OWReadBit(void)
{
	return(OWTouchBit(1));
}

//-----------------------------------------------------------------------------
// OWWriteByte
//-----------------------------------------------------------------------------
void OWWriteByte(unsigned char byte_value)
{
	OWTouchByte(byte_value);
}

//-----------------------------------------------------------------------------
// OWReadByte
//-----------------------------------------------------------------------------
unsigned char OWReadByte(void)
{
	return(OWTouchByte(0xFF));
}

//-----------------------------------------------------------------------------
// docrc8
//-----------------------------------------------------------------------------
unsigned char docrc8(unsigned char value)
{
	crc8 = dscrc_table[crc8 ^ value];
	return(crc8);
}

//-----------------------------------------------------------------------------
// OWSearch
//-----------------------------------------------------------------------------
unsigned char OWSearch(void)
{
	unsigned char id_bit_number;
	unsigned char last_zero, rom_byte_number, search_result;
	unsigned char id_bit, cmp_id_bit;
	unsigned char rom_byte_mask, search_direction;
	
	id_bit_number   = 1;
	last_zero       = 0;
	rom_byte_number = 0;
	rom_byte_mask   = 1;
	search_result   = 0;
	crc8            = 0;
	
	if(!LastDeviceFlag)
	{
		if(!OWTouchReset())
		{
			LastDiscrepancy       = 0;
			LastDeviceFlag        = FALSE;
			LastFamilyDiscrepancy = 0;
			
			return(FALSE);
		}
		OWWriteByte(0xF0);
		do
		{
			id_bit     = OWReadBit();
			cmp_id_bit = OWReadBit();
			
			if((id_bit == 1) && (cmp_id_bit == 1))
				break;
			else
			{
				if(id_bit != cmp_id_bit)
					search_direction = id_bit;
				else
				{
					if(id_bit_number < LastDiscrepancy)
						search_direction = ((ROMCode[rom_byte_number] & rom_byte_mask) > 0);
					else
						search_direction = (id_bit_number == LastDiscrepancy);
					if(search_direction == 0)
					{
						last_zero = id_bit_number;
						if(last_zero < 9)
							LastFamilyDiscrepancy = last_zero;
					}
				}
				if(search_direction == 1)
					ROMCode[rom_byte_number] |= rom_byte_mask;
				else
					ROMCode[rom_byte_number] &= ~rom_byte_mask;
				
				OWWriteBit(search_direction);
				
				id_bit_number++;
				rom_byte_mask <<= 1;
				
				if(rom_byte_mask == 0)
				{
					docrc8(ROMCode[rom_byte_number]);
					rom_byte_number++;
					rom_byte_mask = 1;
				}
			}
		}
		while(rom_byte_number < 8);
		
		if(!((id_bit_number < 65) || (crc8 != 0)))
		{
			LastDiscrepancy = last_zero;
			
			if(LastDiscrepancy == 0)
				LastDeviceFlag = TRUE;
			
			search_result = TRUE;
		}
	}
	
	if(!search_result || !ROMCode[0])
	{
		LastDiscrepancy       = 0;
		LastDeviceFlag        = FALSE;
		LastFamilyDiscrepancy = 0;
		search_result         = FALSE;
	}
	
	return(search_result);
}

//-----------------------------------------------------------------------------
// OWFirst
//-----------------------------------------------------------------------------
unsigned char OWFirst(void)
{
	LastDiscrepancy       = 0;
	LastDeviceFlag        = FALSE;
	LastFamilyDiscrepancy = 0;
	
	return(OWSearch());
}

//-----------------------------------------------------------------------------
// OWNext
//-----------------------------------------------------------------------------
unsigned char OWNext(void)
{
	return(OWSearch());
}
